@@ -36,6 +36,7 @@
 
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkclipboard.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkselection.h>
@@ -57,7 +58,7 @@
 /* default font */
 #define DEFAULT_FONT "-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1"
 
-#define PADDING 2
+#define PADDING 0
 
 /* forward declararations */
 static void zvt_term_init (ZvtTerm *term);
@@ -101,8 +102,8 @@ static int vt_cursor_state(void *user_data, int state);
 static void zvt_term_writemore (gpointer data, gint fd, GdkInputCondition condition);
 static void zvt_term_updated(ZvtTerm *term, int mode);
 static void clone_col(unsigned short **dest, unsigned short *from);
-static void zvt_term_copy_clipboard (ZvtTerm *term);
-static void zvt_term_paste_clipboard (ZvtTerm *term);
+static void zvt_term_real_copy_clipboard (ZvtTerm *term);
+static void zvt_term_real_paste_clipboard (ZvtTerm *term);
 
 /* callbacks from update layer */
 void vt_draw_text(void *user_data, struct vt_line *line, int row, int col, int len, int attr);
@@ -203,7 +204,7 @@ zvt_term_class_init (ZvtTermClass *class)
 		    */
                     gtk_marshal_VOID__POINTER_INT,
                     GTK_TYPE_NONE, 2,
-		    GTK_TYPE_STRING,
+		    GTK_TYPE_POINTER,
 		    GTK_TYPE_INT);
   
   term_signals[GOT_OUTPUT] =
@@ -216,7 +217,7 @@ zvt_term_class_init (ZvtTermClass *class)
 		      */
 		      gtk_marshal_VOID__POINTER_INT,
 		      GTK_TYPE_NONE, 2,
-		      GTK_TYPE_STRING,
+		      GTK_TYPE_POINTER,
 		      GTK_TYPE_INT);
       
   term_signals[COPY_CLIPBOARD] = 
@@ -268,8 +269,8 @@ zvt_term_class_init (ZvtTermClass *class)
   term_class->child_died = zvt_term_child_died;
   term_class->title_changed = zvt_term_title_changed;
   term_class->got_output = zvt_term_got_output;
-  term_class->copy_clipboard = zvt_term_copy_clipboard;
-  term_class->paste_clipboard = zvt_term_paste_clipboard;
+  term_class->copy_clipboard = zvt_term_real_copy_clipboard;
+  term_class->paste_clipboard = zvt_term_real_paste_clipboard;
   term_class->scroll = zvt_term_scroll;
 
   /* Setup default key bindings */
@@ -330,7 +331,7 @@ zvt_term_init (ZvtTerm *term)
 
   /* bitfield flags */
   term->cursor_on = 0;
-  term->cursor_filled = 1;
+  term->cursor_filled = 0;
   term->cursor_blink_state = 0;
   term->scroll_on_keystroke = 0;
   term->scroll_on_output = 0;
@@ -554,9 +555,9 @@ term_force_size(ZvtTerm *term)
     d(printf("forcing term size to %d, %d\n", term->grid_width, term->grid_height));
     d(printf("                   = %d, %d\n",
 	     (term->grid_width * term->charwidth) + 
-	     (GTK_WIDGET(term)->style->klass->xthickness * 2) + PADDING,
+	     (GTK_WIDGET(term)->style->xthickness * 2) + PADDING,
 	     (term->grid_height * term->charheight) + 
-	     (GTK_WIDGET(term)->style->klass->ythickness * 2)));
+	     (GTK_WIDGET(term)->style->ythickness * 2)));
   }
 }
 
@@ -618,11 +619,17 @@ zvt_term_destroy (GtkObject *object)
   term = ZVT_TERM (object);
   zp = _ZVT_PRIVATE(term);
 
-  if (term->timeout_id != -1)
+  if (!zp)
+    return;
+
+  if (term->timeout_id != -1) {
     gtk_timeout_remove(term->timeout_id);
+    term->timeout_id = -1;
+  }
 
   zvt_term_closepty (term);
   vtx_destroy (term->vx);
+  term->vx = NULL;
 
   if (term->font) {
     gdk_font_unref (term->font);
@@ -668,7 +675,7 @@ zvt_term_destroy (GtkObject *object)
 	    g_free(zp->queue_blue);
     
     g_free(zp);
-    gtk_object_set_data (GTK_OBJECT (term), "_zvtprivate", 0);
+   gtk_object_set_data (GTK_OBJECT (term), "_zvtprivate", 0);
   }
 
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -879,9 +886,11 @@ zvt_term_realize (GtkWidget *widget)
   zp->cursor_hand = gdk_cursor_new(GDK_HAND2);
   term->cursor_current = term->cursor_bar;
 
+#if 0 /* this happens on focus_in now */
   /* setup blinking cursor */
   if (term->blink_enabled)
     term->timeout_id = gtk_timeout_add(500, zvt_term_cursor_blink, term);
+#endif
 
   /* setup scrolling gc */
   term->scroll_gc = gdk_gc_new (widget->window);
@@ -929,7 +938,7 @@ zvt_term_realize (GtkWidget *widget)
       g_warning("Can't create input context.");
     }
   }
-#endif UNDEF
+#endif /* UNDEF */
   /* if we have a delayed background set, set it here? */
   if (zp->background_queue) {
     zvt_term_background_load(term, zp->background_queue);
@@ -1164,7 +1173,6 @@ zvt_term_size_allocate (GtkWidget     *widget,
       grid_height = MAX(term_height / term->charheight, 1);
       if (grid_width != term->charwidth
 	  || grid_height != term->charheight) {
-
 	/* turn off the selection */
 	term->vx->selstartx = term->vx->selendx;
 	term->vx->selstarty = term->vx->selendy;
@@ -2654,30 +2662,36 @@ zvt_term_title_changed (ZvtTerm *term, char *str, VTTITLE_TYPE type)
   /* perhaps we should do something here? */
 }
 
-static void 
+static void
+zvt_term_real_copy_clipboard (ZvtTerm *term)
+{
+  if (term->vx->selection_size != 0)
+    gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
+			    (char *) term->vx->selection_data,
+			    term->vx->selection_size);
+}
+
+static void
+zvt_term_real_paste_clipboard (ZvtTerm *term)
+{
+  gtk_selection_convert (GTK_WIDGET (term),
+			 gdk_atom_intern ("CLIPBOARD", FALSE),
+			 GDK_SELECTION_TYPE_STRING,
+			 gtk_get_current_event_time ());
+}
+
+void
 zvt_term_copy_clipboard (ZvtTerm *term)
 {
-  if(term->vx->selection_size != 0){	
-	  if(gtk_selection_owner_set (GTK_WIDGET(term),
-	     gdk_atom_intern ("CLIPBOARD", FALSE),
-	     GDK_CURRENT_TIME)){
- 	    d(printf("ownership claim succeded\n"));
-	  }else{
- 	    d(printf("owbership claim failed\n"));
- 	  }
-  }
+  g_signal_emit (term, term_signals[COPY_CLIPBOARD], 0);
 }
 
-static void 
+void
 zvt_term_paste_clipboard (ZvtTerm *term)
 {
- gtk_selection_convert (GTK_WIDGET (term),
-			gdk_atom_intern ("CLIPBOARD", FALSE),
-			GDK_SELECTION_TYPE_STRING,
-			GDK_CURRENT_TIME);
+  g_signal_emit (term, term_signals[PASTE_CLIPBOARD], 0);
 }
 
-
 /**
  * zvt_term_match_add:
  * @term: An initialised &ZvtTerm.
@@ -3368,7 +3382,7 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
   if (fill >= 17) {
     gc=term->back_gc;
   } else {
-#endif UNDEF
+#endif /* UNDEF */
     gc=term->fore_gc;
     if (term->fore_last != fill) {
       GdkColor pen;
@@ -3379,7 +3393,7 @@ vt_scroll_area(void *user_data, int firstrow, int count, int offset, int fill)
     }
 #ifdef UNDEF
   }
-#endif UNDEF
+#endif /* UNDEF */
 
   /* this does a 'scrolling' pixmap */
   if (zp->background) {
